@@ -1,4 +1,4 @@
-/*	$OpenBSD: mrt.c,v 1.114 2023/04/19 09:03:00 claudio Exp $ */
+/*	$OpenBSD: mrt.c,v 1.123 2024/12/16 16:10:10 claudio Exp $ */
 
 /*
  * Copyright (c) 2003, 2004 Claudio Jeker <claudio@openbsd.org>
@@ -34,55 +34,19 @@
 #include "mrt.h"
 #include "log.h"
 
-int mrt_attr_dump(struct ibuf *, struct rde_aspath *, struct rde_community *,
-    struct bgpd_addr *, int);
-int mrt_dump_entry_mp(struct mrt *, struct prefix *, uint16_t,
-    struct rde_peer*);
-int mrt_dump_entry(struct mrt *, struct prefix *, uint16_t, struct rde_peer*);
-int mrt_dump_entry_v2(struct mrt *, struct rib_entry *, uint32_t);
-int mrt_dump_peer(struct ibuf *, struct rde_peer *);
-int mrt_dump_hdr_se(struct ibuf **, struct peer *, uint16_t, uint16_t,
-    uint32_t, int);
-int mrt_dump_hdr_rde(struct ibuf **, uint16_t type, uint16_t, uint32_t);
-int mrt_open(struct mrt *, time_t);
-
-#define DUMP_BYTE(x, b)							\
-	do {								\
-		u_char		t = (b);				\
-		if (ibuf_add((x), &t, sizeof(t)) == -1) {		\
-			log_warn("mrt_dump1: ibuf_add error");		\
-			goto fail;					\
-		}							\
-	} while (0)
-
-#define DUMP_SHORT(x, s)						\
-	do {								\
-		uint16_t	t;					\
-		t = htons((s));						\
-		if (ibuf_add((x), &t, sizeof(t)) == -1) {		\
-			log_warn("mrt_dump2: ibuf_add error");		\
-			goto fail;					\
-		}							\
-	} while (0)
-
-#define DUMP_LONG(x, l)							\
-	do {								\
-		uint32_t	t;					\
-		t = htonl((l));						\
-		if (ibuf_add((x), &t, sizeof(t)) == -1) {		\
-			log_warn("mrt_dump3: ibuf_add error");		\
-			goto fail;					\
-		}							\
-	} while (0)
-
-#define DUMP_NLONG(x, l)						\
-	do {								\
-		uint32_t	t = (l);				\
-		if (ibuf_add((x), &t, sizeof(t)) == -1) {		\
-			log_warn("mrt_dump4: ibuf_add error");		\
-			goto fail;					\
-		}							\
-	} while (0)
+static int	mrt_attr_dump(struct ibuf *, struct rde_aspath *,
+		    struct rde_community *, struct bgpd_addr *, int);
+static int	mrt_dump_entry_mp(struct mrt *, struct prefix *, uint16_t,
+		    struct rde_peer*);
+static int	mrt_dump_entry(struct mrt *, struct prefix *, uint16_t,
+		    struct rde_peer*);
+static int	mrt_dump_entry_v2(struct mrt *, struct rib_entry *, uint32_t);
+static int	mrt_dump_peer(struct ibuf *, struct rde_peer *);
+static int	mrt_dump_hdr_se(struct ibuf **, struct peer *, uint16_t,
+		    uint16_t, uint32_t, int);
+static int	mrt_dump_hdr_rde(struct ibuf **, uint16_t type, uint16_t,
+		    uint32_t);
+static int	mrt_open(struct mrt *, time_t);
 
 #define RDEIDX		0
 #define SEIDX		1
@@ -92,33 +56,27 @@ int mrt_open(struct mrt *, time_t);
 			)
 
 static uint8_t
-mrt_update_msg_guess_aid(uint8_t *pkg, uint16_t pkglen)
+mrt_update_msg_guess_aid(struct ibuf *pkg)
 {
+	struct ibuf buf;
 	uint16_t wlen, alen, len, afi;
-	uint8_t type, aid;
+	uint8_t type, flags, aid, safi;
 
-	pkg += MSGSIZE_HEADER;
-	pkglen -= MSGSIZE_HEADER;
+	ibuf_from_ibuf(&buf, pkg);
 
-	if (pkglen < 4)
+	if (ibuf_skip(&buf, MSGSIZE_HEADER) == -1 ||
+	    ibuf_get_n16(&buf, &wlen) == -1)
 		goto bad;
 
-	memcpy(&wlen, pkg, 2);
-	wlen = ntohs(wlen);
-	pkg += 2;
-	pkglen -= 2;
-
 	if (wlen > 0) {
 		/* UPDATE has withdraw routes, therefore IPv4 */
 		return AID_INET;
 	}
 
-	memcpy(&alen, pkg, 2);
-	alen = ntohs(alen);
-	pkg += 2;
-	pkglen -= 2;
+	if (ibuf_get_n16(&buf, &alen) == -1)
+		goto bad;
 
-	if (alen < pkglen) {
+	if (alen < ibuf_size(&buf)) {
 		/* UPDATE has NLRI prefixes, therefore IPv4 */
 		return AID_INET;
 	}
@@ -129,42 +87,37 @@ mrt_update_msg_guess_aid(uint8_t *pkg, uint16_t pkglen)
 	}
 
 	/* bad attribute length */
-	if (alen > pkglen)
+	if (alen > ibuf_size(&buf))
 		goto bad;
 
 	/* try to extract AFI/SAFI from the MP attributes */
-	while (alen > 0) {
-		if (alen < 3)
+	while (ibuf_size(&buf) > 0) {
+		if (ibuf_get_n8(&buf, &flags) == -1 ||
+		    ibuf_get_n8(&buf, &type) == -1)
 			goto bad;
-		type = pkg[1];
-		if (pkg[0] & ATTR_EXTLEN) {
-			if (alen < 4)
+		if (flags & ATTR_EXTLEN) {
+			if (ibuf_get_n16(&buf, &len) == -1)
 				goto bad;
-			memcpy(&len, pkg + 2, 2);
-			len = ntohs(len);
-			pkg += 4;
-			alen -= 4;
 		} else {
-			len = pkg[2];
-			pkg += 3;
-			alen -= 3;
+			uint8_t tmp;
+			if (ibuf_get_n8(&buf, &tmp) == -1)
+				goto bad;
+			len = tmp;
 		}
-		if (len > alen)
+		if (len > ibuf_size(&buf))
 			goto bad;
 
 		if (type == ATTR_MP_REACH_NLRI ||
 		    type == ATTR_MP_UNREACH_NLRI) {
-			if (alen < 3)
+			if (ibuf_get_n16(&buf, &afi) == -1 ||
+			    ibuf_get_n8(&buf, &safi) == -1)
 				goto bad;
-			memcpy(&afi, pkg, 2);
-			afi = ntohs(afi);
-			if (afi2aid(afi, pkg[2], &aid) == -1)
+			if (afi2aid(afi, safi, &aid) == -1)
 				goto bad;
 			return aid;
 		}
-
-		pkg += len;
-		alen -= len;
+		if (ibuf_skip(&buf, len) == -1)
+			goto bad;
 	}
 
 bad:
@@ -172,8 +125,8 @@ bad:
 }
 
 static uint16_t
-mrt_bgp_msg_subtype(struct mrt *mrt, void *pkg, uint16_t pkglen,
-    struct peer *peer, enum msg_type msgtype, int in)
+mrt_bgp_msg_subtype(struct mrt *mrt, struct ibuf *pkg, struct peer *peer,
+    enum msg_type msgtype, int in)
 {
 	uint16_t subtype = BGP4MP_MESSAGE;
 	uint8_t aid, mask;
@@ -181,7 +134,7 @@ mrt_bgp_msg_subtype(struct mrt *mrt, void *pkg, uint16_t pkglen,
 	if (peer->capa.neg.as4byte)
 		subtype = BGP4MP_MESSAGE_AS4;
 
-	if (msgtype != UPDATE)
+	if (msgtype != MSG_UPDATE)
 		return subtype;
 
 	/*
@@ -194,7 +147,7 @@ mrt_bgp_msg_subtype(struct mrt *mrt, void *pkg, uint16_t pkglen,
 	mask = in ? CAPA_AP_RECV : CAPA_AP_SEND;
 	/* only guess if add-path could be active */
 	if (peer->capa.neg.add_path[0] & mask) {
-		aid = mrt_update_msg_guess_aid(pkg, pkglen);
+		aid = mrt_update_msg_guess_aid(pkg);
 		if (aid != AID_UNSPEC &&
 		    (peer->capa.neg.add_path[aid] & mask)) {
 			if (peer->capa.neg.as4byte)
@@ -208,8 +161,8 @@ mrt_bgp_msg_subtype(struct mrt *mrt, void *pkg, uint16_t pkglen,
 }
 
 void
-mrt_dump_bgp_msg(struct mrt *mrt, void *pkg, uint16_t pkglen,
-    struct peer *peer, enum msg_type msgtype)
+mrt_dump_bgp_msg(struct mrt *mrt, struct ibuf *pkg, struct peer *peer,
+    enum msg_type msgtype)
 {
 	struct ibuf	*buf;
 	int		 in = 0;
@@ -219,19 +172,21 @@ mrt_dump_bgp_msg(struct mrt *mrt, void *pkg, uint16_t pkglen,
 	if (mrt->type == MRT_ALL_IN || mrt->type == MRT_UPDATE_IN)
 		in = 1;
 
-	subtype = mrt_bgp_msg_subtype(mrt, pkg, pkglen, peer, msgtype, in);
+	subtype = mrt_bgp_msg_subtype(mrt, pkg, peer, msgtype, in);
 
 	if (mrt_dump_hdr_se(&buf, peer, MSG_PROTOCOL_BGP4MP_ET, subtype,
-	    pkglen, in) == -1)
-		return;
+	    ibuf_size(pkg), in) == -1)
+		goto fail;
 
-	if (ibuf_add(buf, pkg, pkglen) == -1) {
-		log_warn("mrt_dump_bgp_msg: ibuf_add error");
-		ibuf_free(buf);
-		return;
-	}
+	if (ibuf_add_ibuf(buf, pkg) == -1)
+		goto fail;
 
-	ibuf_close(&mrt->wbuf, buf);
+	ibuf_close(mrt->wbuf, buf);
+	return;
+
+fail:
+	log_warn("%s: ibuf error", __func__);
+	ibuf_free(buf);
 }
 
 void
@@ -246,19 +201,22 @@ mrt_dump_state(struct mrt *mrt, uint16_t old_state, uint16_t new_state,
 
 	if (mrt_dump_hdr_se(&buf, peer, MSG_PROTOCOL_BGP4MP_ET, subtype,
 	    2 * sizeof(short), 0) == -1)
-		return;
+		goto fail;
 
-	DUMP_SHORT(buf, old_state);
-	DUMP_SHORT(buf, new_state);
+	if (ibuf_add_n16(buf, old_state) == -1)
+		goto fail;
+	if (ibuf_add_n16(buf, new_state) == -1)
+		goto fail;
 
-	ibuf_close(&mrt->wbuf, buf);
+	ibuf_close(mrt->wbuf, buf);
 	return;
 
 fail:
+	log_warn("%s: ibuf error", __func__);
 	ibuf_free(buf);
 }
 
-int
+static int
 mrt_attr_dump(struct ibuf *buf, struct rde_aspath *a, struct rde_community *c,
     struct bgpd_addr *nexthop, int v2)
 {
@@ -309,7 +267,9 @@ mrt_attr_dump(struct ibuf *buf, struct rde_aspath *a, struct rde_community *c,
 		return (-1);
 
 	/* communities */
-	if (community_writebuf(buf, c) == -1)
+	if (community_writebuf(c, ATTR_COMMUNITIES, 0, buf) == -1 ||
+	    community_writebuf(c, ATTR_EXT_COMMUNITIES, 0, buf) == -1 ||
+	    community_writebuf(c, ATTR_LARGE_COMMUNITIES, 0, buf) == -1)
 		return (-1);
 
 	/* dump all other path attributes without modification */
@@ -328,38 +288,46 @@ mrt_attr_dump(struct ibuf *buf, struct rde_aspath *a, struct rde_community *c,
 			return (-1);
 		if (!v2) {
 			if (aid2afi(nexthop->aid, &afi, &safi))
-				return (-1);
-			DUMP_SHORT(nhbuf, afi);
-			DUMP_BYTE(nhbuf, safi);
+				goto fail;
+			if (ibuf_add_n16(nhbuf, afi) == -1)
+				goto fail;
+			if (ibuf_add_n8(nhbuf, safi) == -1)
+				goto fail;
 		}
 		switch (nexthop->aid) {
 		case AID_INET6:
-			DUMP_BYTE(nhbuf, sizeof(struct in6_addr));
+			if (ibuf_add_n8(nhbuf, sizeof(struct in6_addr)) == -1)
+				goto fail;
 			if (ibuf_add(nhbuf, &nexthop->v6,
 			    sizeof(struct in6_addr)) == -1)
 				goto fail;
 			break;
 		case AID_VPN_IPv4:
-			DUMP_BYTE(nhbuf, sizeof(uint64_t) +
-			    sizeof(struct in_addr));
-			DUMP_NLONG(nhbuf, 0);	/* set RD to 0 */
-			DUMP_NLONG(nhbuf, 0);
-			DUMP_NLONG(nhbuf, nexthop->v4.s_addr);
+			if (ibuf_add_n8(nhbuf, sizeof(uint64_t) +
+			    sizeof(struct in_addr)) == -1)
+				goto fail;
+			if (ibuf_add_n64(nhbuf, 0) == -1) /* set RD to 0 */
+				goto fail;
+			if (ibuf_add(nhbuf, &nexthop->v4,
+			    sizeof(nexthop->v4)) == -1)
+				goto fail;
 			break;
 		case AID_VPN_IPv6:
-			DUMP_BYTE(nhbuf, sizeof(uint64_t) +
-			    sizeof(struct in6_addr));
-			DUMP_NLONG(nhbuf, 0);	/* set RD to 0 */
-			DUMP_NLONG(nhbuf, 0);
-			if (ibuf_add(nhbuf, &nexthop->v6,
+			if (ibuf_add_n8(nhbuf, sizeof(uint64_t) +
 			    sizeof(struct in6_addr)) == -1)
 				goto fail;
+			if (ibuf_add_n64(nhbuf, 0) == -1) /* set RD to 0 */
+				goto fail;
+			if (ibuf_add(nhbuf, &nexthop->v6,
+			    sizeof(nexthop->v6)) == -1)
+				goto fail;
 			break;
 		}
 		if (!v2)
-			DUMP_BYTE(nhbuf, 0);
+			if (ibuf_add_n8(nhbuf, 0) == -1)
+				goto fail;
 		if (attr_writebuf(buf, ATTR_OPTIONAL, ATTR_MP_REACH_NLRI,
-		    nhbuf->buf, ibuf_size(nhbuf)) == -1) {
+		    ibuf_data(nhbuf), ibuf_size(nhbuf)) == -1) {
 fail:
 			ibuf_free(nhbuf);
 			return (-1);
@@ -381,7 +349,7 @@ fail:
 	return (0);
 }
 
-int
+static int
 mrt_dump_entry_mp(struct mrt *mrt, struct prefix *p, uint16_t snum,
     struct rde_peer *peer)
 {
@@ -397,22 +365,21 @@ mrt_dump_entry_mp(struct mrt *mrt, struct prefix *p, uint16_t snum,
 	}
 
 	if (mrt_attr_dump(buf, prefix_aspath(p), prefix_communities(p),
-	    NULL, 0) == -1) {
-		log_warnx("mrt_dump_entry_mp: mrt_attr_dump error");
+	    NULL, 0) == -1)
 		goto fail;
-	}
 	len = ibuf_size(buf);
 
 	if ((h2buf = ibuf_dynamic(MRT_BGP4MP_IPv4_HEADER_SIZE +
 	    MRT_BGP4MP_IPv4_ENTRY_SIZE, MRT_BGP4MP_IPv6_HEADER_SIZE +
-	    MRT_BGP4MP_IPv6_ENTRY_SIZE + MRT_BGP4MP_MAX_PREFIXLEN)) == NULL) {
-		log_warn("mrt_dump_entry_mp: ibuf_dynamic");
+	    MRT_BGP4MP_IPv6_ENTRY_SIZE + MRT_BGP4MP_MAX_PREFIXLEN)) == NULL)
 		goto fail;
-	}
 
-	DUMP_SHORT(h2buf, peer->conf.local_short_as);
-	DUMP_SHORT(h2buf, peer->short_as);
-	DUMP_SHORT(h2buf, /* ifindex */ 0);
+	if (ibuf_add_n16(h2buf, peer->conf.local_short_as) == -1)
+		goto fail;
+	if (ibuf_add_n16(h2buf, peer->short_as) == -1)
+		goto fail;
+	if (ibuf_add_n16(h2buf, /* ifindex */ 0) == -1)
+		goto fail;
 
 	/* XXX is this for peer self? */
 	aid = peer->remote_addr.aid == AID_UNSPEC ? p->pt->aid :
@@ -420,30 +387,37 @@ mrt_dump_entry_mp(struct mrt *mrt, struct prefix *p, uint16_t snum,
 	switch (aid) {
 	case AID_INET:
 	case AID_VPN_IPv4:
-		DUMP_SHORT(h2buf, AFI_IPv4);
-		DUMP_NLONG(h2buf, peer->local_v4_addr.v4.s_addr);
-		DUMP_NLONG(h2buf, peer->remote_addr.v4.s_addr);
+		if (ibuf_add_n16(h2buf, AFI_IPv4) == -1)
+			goto fail;
+		if (ibuf_add(h2buf, &peer->local_v4_addr.v4,
+		    sizeof(peer->local_v4_addr.v4)) == -1 ||
+		    ibuf_add(h2buf, &peer->remote_addr.v4,
+		    sizeof(peer->remote_addr.v4)) == -1)
+			goto fail;
 		break;
 	case AID_INET6:
 	case AID_VPN_IPv6:
-		DUMP_SHORT(h2buf, AFI_IPv6);
+		if (ibuf_add_n16(h2buf, AFI_IPv6) == -1)
+			goto fail;
 		if (ibuf_add(h2buf, &peer->local_v6_addr.v6,
-		    sizeof(struct in6_addr)) == -1 ||
+		    sizeof(peer->local_v6_addr.v6)) == -1 ||
 		    ibuf_add(h2buf, &peer->remote_addr.v6,
-		    sizeof(struct in6_addr)) == -1) {
-			log_warn("mrt_dump_entry_mp: ibuf_add error");
+		    sizeof(peer->remote_addr.v6)) == -1)
 			goto fail;
-		}
 		break;
 	default:
 		log_warnx("king bula found new AF %d in %s", aid, __func__);
 		goto fail;
 	}
 
-	DUMP_SHORT(h2buf, 0);		/* view */
-	DUMP_SHORT(h2buf, 1);		/* status */
+	if (ibuf_add_n16(h2buf, 0) == -1)		/* view */
+		goto fail;
+	if (ibuf_add_n16(h2buf, 1) == -1)		/* status */
+		goto fail;
 	/* originated timestamp */
-	DUMP_LONG(h2buf, time(NULL) - (getmonotime() - p->lastchange));
+	if (ibuf_add_n32(h2buf, time(NULL) - (getmonotime() -
+	    p->lastchange)) == -1)
+		goto fail;
 
 	n = prefix_nexthop(p);
 	if (n == NULL) {
@@ -455,83 +429,100 @@ mrt_dump_entry_mp(struct mrt *mrt, struct prefix *p, uint16_t snum,
 
 	switch (p->pt->aid) {
 	case AID_INET:
-		DUMP_SHORT(h2buf, AFI_IPv4);	/* afi */
-		DUMP_BYTE(h2buf, SAFI_UNICAST);	/* safi */
-		DUMP_BYTE(h2buf, 4);		/* nhlen */
-		DUMP_NLONG(h2buf, nh->v4.s_addr);	/* nexthop */
+		if (ibuf_add_n16(h2buf, AFI_IPv4) == -1)	/* afi */
+			goto fail;
+		if (ibuf_add_n8(h2buf, SAFI_UNICAST) == -1)	/* safi */
+			goto fail;
+		if (ibuf_add_n8(h2buf, 4) == -1)		/* nhlen */
+			goto fail;
+		if (ibuf_add(h2buf, &nh->v4, sizeof(nh->v4)) == -1)
+			goto fail;
 		break;
 	case AID_INET6:
-		DUMP_SHORT(h2buf, AFI_IPv6);	/* afi */
-		DUMP_BYTE(h2buf, SAFI_UNICAST);	/* safi */
-		DUMP_BYTE(h2buf, 16);		/* nhlen */
-		if (ibuf_add(h2buf, &nh->v6, sizeof(struct in6_addr)) == -1) {
-			log_warn("mrt_dump_entry_mp: ibuf_add error");
+		if (ibuf_add_n16(h2buf, AFI_IPv6) == -1)	/* afi */
+			goto fail;
+		if (ibuf_add_n8(h2buf, SAFI_UNICAST) == -1)	/* safi */
+			goto fail;
+		if (ibuf_add_n8(h2buf, 16) == -1)		/* nhlen */
+			goto fail;
+		if (ibuf_add(h2buf, &nh->v6, sizeof(nh->v6)) == -1)
 			goto fail;
-		}
 		break;
 	case AID_VPN_IPv4:
-		DUMP_SHORT(h2buf, AFI_IPv4);	/* afi */
-		DUMP_BYTE(h2buf, SAFI_MPLSVPN);	/* safi */
-		DUMP_BYTE(h2buf, sizeof(uint64_t) + sizeof(struct in_addr));
-		DUMP_NLONG(h2buf, 0);	/* set RD to 0 */
-		DUMP_NLONG(h2buf, 0);
-		DUMP_NLONG(h2buf, nh->v4.s_addr);	/* nexthop */
+		if (ibuf_add_n16(h2buf, AFI_IPv4) == -1)	/* afi */
+			goto fail;
+		if (ibuf_add_n8(h2buf, SAFI_MPLSVPN) == -1)	/* safi */
+			goto fail;
+		if (ibuf_add_n8(h2buf, sizeof(uint64_t) +
+		    sizeof(struct in_addr)) == -1)
+			goto fail;
+		if (ibuf_add_n64(h2buf, 0) == -1)	/* set RD to 0 */
+			goto fail;
+		if (ibuf_add(h2buf, &nh->v4, sizeof(nh->v4)) == -1)
+			goto fail;
 		break;
 	case AID_VPN_IPv6:
-		DUMP_SHORT(h2buf, AFI_IPv6);	/* afi */
-		DUMP_BYTE(h2buf, SAFI_MPLSVPN);	/* safi */
-		DUMP_BYTE(h2buf, sizeof(uint64_t) + sizeof(struct in6_addr));
-		DUMP_NLONG(h2buf, 0);	/* set RD to 0 */
-		DUMP_NLONG(h2buf, 0);
-		if (ibuf_add(h2buf, &nh->v6, sizeof(struct in6_addr)) == -1) {
-			log_warn("mrt_dump_entry_mp: ibuf_add error");
+		if (ibuf_add_n16(h2buf, AFI_IPv6) == -1)	/* afi */
+			goto fail;
+		if (ibuf_add_n8(h2buf, SAFI_MPLSVPN) == -1)	/* safi */
+			goto fail;
+		if (ibuf_add_n8(h2buf, sizeof(uint64_t) +
+		    sizeof(struct in6_addr)) == -1)
+			goto fail;
+		if (ibuf_add_n64(h2buf, 0) == -1)	/* set RD to 0 */
+			goto fail;
+		if (ibuf_add(h2buf, &nh->v6, sizeof(nh->v6)) == -1)
 			goto fail;
-		}
 		break;
 	case AID_FLOWSPECv4:
 	case AID_FLOWSPECv6:
-		if (p->pt->aid == AID_FLOWSPECv4)
-			DUMP_SHORT(h2buf, AFI_IPv4);	/* afi */
-		else
-			DUMP_SHORT(h2buf, AFI_IPv6);	/* afi */
-		DUMP_BYTE(h2buf, SAFI_FLOWSPEC);	/* safi */
-		DUMP_BYTE(h2buf, 0);			/* nhlen */
+		if (p->pt->aid == AID_FLOWSPECv4) {
+			if (ibuf_add_n16(h2buf, AFI_IPv4) == -1) /* afi */
+				goto fail;
+		} else {
+			if (ibuf_add_n16(h2buf, AFI_IPv6) == -1) /* afi */
+				goto fail;
+		}
+		if (ibuf_add_n8(h2buf, SAFI_FLOWSPEC) == -1)	/* safi */
+			goto fail;
+		if (ibuf_add_n8(h2buf, 0) == -1)		/* nhlen */
+			goto fail;
 		break;
 	default:
 		log_warnx("king bula found new AF in %s", __func__);
 		goto fail;
 	}
 
-	if (pt_writebuf(h2buf, p->pt) == -1) {
-		log_warnx("%s: pt_writebuf error", __func__);
+	if (pt_writebuf(h2buf, p->pt, 0, 0, 0) == -1)
 		goto fail;
-	}
 
-	DUMP_SHORT(h2buf, len);
+	if (ibuf_add_n16(h2buf, len) == -1)
+		goto fail;
 	len += ibuf_size(h2buf);
 
 	if (mrt_dump_hdr_rde(&hbuf, MSG_PROTOCOL_BGP4MP, BGP4MP_ENTRY,
 	    len) == -1)
 		goto fail;
 
-	ibuf_close(&mrt->wbuf, hbuf);
-	ibuf_close(&mrt->wbuf, h2buf);
-	ibuf_close(&mrt->wbuf, buf);
+	ibuf_close(mrt->wbuf, hbuf);
+	ibuf_close(mrt->wbuf, h2buf);
+	ibuf_close(mrt->wbuf, buf);
 
 	return (len + MRT_HEADER_SIZE);
 
 fail:
+	log_warn("%s: ibuf error", __func__);
 	ibuf_free(hbuf);
 	ibuf_free(h2buf);
 	ibuf_free(buf);
 	return (-1);
 }
 
-int
+static int
 mrt_dump_entry(struct mrt *mrt, struct prefix *p, uint16_t snum,
     struct rde_peer *peer)
 {
-	struct ibuf	*buf, *hbuf;
+	struct ibuf	*buf, *hbuf = NULL;
 	struct nexthop	*nexthop;
 	struct bgpd_addr addr, *nh;
 	size_t		 len;
@@ -556,59 +547,63 @@ mrt_dump_entry(struct mrt *mrt, struct prefix *p, uint16_t snum,
 	} else
 		nh = &nexthop->exit_nexthop;
 	if (mrt_attr_dump(buf, prefix_aspath(p), prefix_communities(p),
-	    nh, 0) == -1) {
-		log_warnx("mrt_dump_entry: mrt_attr_dump error");
-		ibuf_free(buf);
-		return (-1);
-	}
+	    nh, 0) == -1)
+		goto fail;
+
 	len = ibuf_size(buf);
 	aid2afi(p->pt->aid, &subtype, &dummy);
-	if (mrt_dump_hdr_rde(&hbuf, MSG_TABLE_DUMP, subtype, len) == -1) {
-		ibuf_free(buf);
-		return (-1);
-	}
+	if (mrt_dump_hdr_rde(&hbuf, MSG_TABLE_DUMP, subtype, len) == -1)
+		goto fail;
 
-	DUMP_SHORT(hbuf, 0);
-	DUMP_SHORT(hbuf, snum);
+	if (ibuf_add_n16(hbuf, 0) == -1)
+		goto fail;
+	if (ibuf_add_n16(hbuf, snum) == -1)
+		goto fail;
 
 	pt_getaddr(p->pt, &addr);
 	switch (p->pt->aid) {
 	case AID_INET:
-		DUMP_NLONG(hbuf, addr.v4.s_addr);
+		if (ibuf_add(hbuf, &addr.v4, sizeof(addr.v4)) == -1)
+			goto fail;
 		break;
 	case AID_INET6:
-		if (ibuf_add(hbuf, &addr.v6, sizeof(struct in6_addr)) == -1) {
-			log_warn("mrt_dump_entry: ibuf_add error");
+		if (ibuf_add(hbuf, &addr.v6, sizeof(addr.v6)) == -1)
 			goto fail;
-		}
 		break;
 	}
-	DUMP_BYTE(hbuf, p->pt->prefixlen);
+	if (ibuf_add_n8(hbuf, p->pt->prefixlen) == -1)
+		goto fail;
 
-	DUMP_BYTE(hbuf, 1);		/* state */
+	if (ibuf_add_n8(hbuf, 1) == -1)		/* state */
+		goto fail;
 	/* originated timestamp */
-	DUMP_LONG(hbuf, time(NULL) - (getmonotime() - p->lastchange));
+	if (ibuf_add_n32(hbuf, time(NULL) - (getmonotime() -
+	    p->lastchange)) == -1)
+		goto fail;
 	switch (p->pt->aid) {
 	case AID_INET:
-		DUMP_NLONG(hbuf, peer->remote_addr.v4.s_addr);
+		if (ibuf_add(hbuf, &peer->remote_addr.v4,
+		    sizeof(peer->remote_addr.v4)) == -1)
+			goto fail;
 		break;
 	case AID_INET6:
 		if (ibuf_add(hbuf, &peer->remote_addr.v6,
-		    sizeof(struct in6_addr)) == -1) {
-			log_warn("mrt_dump_entry: ibuf_add error");
+		    sizeof(peer->remote_addr.v6)) == -1)
 			goto fail;
-		}
 		break;
 	}
-	DUMP_SHORT(hbuf, peer->short_as);
-	DUMP_SHORT(hbuf, len);
+	if (ibuf_add_n16(hbuf, peer->short_as) == -1)
+		goto fail;
+	if (ibuf_add_n16(hbuf, len) == -1)
+		goto fail;
 
-	ibuf_close(&mrt->wbuf, hbuf);
-	ibuf_close(&mrt->wbuf, buf);
+	ibuf_close(mrt->wbuf, hbuf);
+	ibuf_close(mrt->wbuf, buf);
 
 	return (len + MRT_HEADER_SIZE);
 
 fail:
+	log_warn("%s: ibuf error", __func__);
 	ibuf_free(hbuf);
 	ibuf_free(buf);
 	return (-1);
@@ -619,9 +614,9 @@ mrt_dump_entry_v2_rib(struct rib_entry *re, struct ibuf **nb, struct ibuf **apb,
     uint16_t *np, uint16_t *app)
 {
 	struct bgpd_addr addr;
-	struct ibuf *buf, **bp;
+	struct ibuf *buf = NULL, **bp;
+	struct ibuf *tbuf = NULL;
 	struct prefix *p;
-	size_t len;
 	int addpath;
 
 	*np = 0;
@@ -630,7 +625,6 @@ mrt_dump_entry_v2_rib(struct rib_entry *re, struct ibuf **nb, struct ibuf **apb,
 	TAILQ_FOREACH(p, &re->prefix_h, entry.list.rib) {
 		struct nexthop		*nexthop;
 		struct bgpd_addr	*nh;
-		struct ibuf		*tbuf;
 
 		addpath = peer_has_add_path(prefix_peer(p), re->prefix->aid,
 		    CAPA_AP_RECV);
@@ -643,10 +637,8 @@ mrt_dump_entry_v2_rib(struct rib_entry *re, struct ibuf **nb, struct ibuf **apb,
 			*np += 1;
 		}
 		if ((buf = *bp) == NULL) {
-			if ((buf = ibuf_dynamic(0, UINT_MAX)) == NULL) {
-				log_warn("%s: ibuf_dynamic", __func__);
+			if ((buf = ibuf_dynamic(0, UINT_MAX)) == NULL)
 				goto fail;
-			}
 			*bp = buf;
 		}
 
@@ -658,41 +650,39 @@ mrt_dump_entry_v2_rib(struct rib_entry *re, struct ibuf **nb, struct ibuf **apb,
 		} else
 			nh = &nexthop->exit_nexthop;
 
-		DUMP_SHORT(buf, prefix_peer(p)->mrt_idx);
+		if (ibuf_add_n16(buf, prefix_peer(p)->mrt_idx) == -1)
+			goto fail;
 		/* originated timestamp */
-		DUMP_LONG(buf, time(NULL) - (getmonotime() - p->lastchange));
+		if (ibuf_add_n32(buf, time(NULL) - (getmonotime() -
+		    p->lastchange)) == -1)
+			goto fail;
 
 		/* RFC8050: path-id if add-path is used */
 		if (addpath)
-			DUMP_LONG(buf, p->path_id);
+			if (ibuf_add_n32(buf, p->path_id) == -1)
+				goto fail;
 
-		if ((tbuf = ibuf_dynamic(0, MAX_PKTSIZE)) == NULL) {
-			log_warn("%s: ibuf_dynamic", __func__);
+		if ((tbuf = ibuf_dynamic(0, MAX_PKTSIZE)) == NULL)
 			goto fail;
-		}
 		if (mrt_attr_dump(tbuf, prefix_aspath(p), prefix_communities(p),
-		    nh, 1) == -1) {
-			log_warnx("%s: mrt_attr_dump error", __func__);
-			ibuf_free(tbuf);
+		    nh, 1) == -1)
 			goto fail;
-		}
-		len = ibuf_size(tbuf);
-		DUMP_SHORT(buf, (uint16_t)len);
-		if (ibuf_add(buf, tbuf->buf, len) == -1) {
-			log_warn("%s: ibuf_add error", __func__);
-			ibuf_free(tbuf);
+		if (ibuf_add_n16(buf, ibuf_size(tbuf)) == -1)
+			goto fail;
+		if (ibuf_add_ibuf(buf, tbuf) == -1)
 			goto fail;
-		}
 		ibuf_free(tbuf);
+		tbuf = NULL;
 	}
 
 	return 0;
 
 fail:
+	ibuf_free(tbuf);
 	return -1;
 }
 
-int
+static int
 mrt_dump_entry_v2(struct mrt *mrt, struct rib_entry *re, uint32_t snum)
 {
 	struct ibuf	*hbuf = NULL, *nbuf = NULL, *apbuf = NULL, *pbuf;
@@ -726,15 +716,15 @@ mrt_dump_entry_v2(struct mrt *mrt, struct rib_entry *re, uint32_t snum)
 		aid2afi(re->prefix->aid, &afi, &safi);
 
 		/* first add 3-bytes AFI/SAFI */
-		DUMP_SHORT(pbuf, afi);
-		DUMP_BYTE(pbuf, safi);
+		if (ibuf_add_n16(pbuf, afi) == -1)
+			goto fail;
+		if (ibuf_add_n8(pbuf, safi) == -1)
+			goto fail;
 		break;
 	}
 
-	if (pt_writebuf(pbuf, re->prefix) == -1) {
-		log_warnx("%s: pt_writebuf error", __func__);
+	if (pt_writebuf(pbuf, re->prefix, 0, 0, 0) == -1)
 		goto fail;
-	}
 
 	hlen = sizeof(snum) + sizeof(nump) + ibuf_size(pbuf);
 
@@ -747,15 +737,15 @@ mrt_dump_entry_v2(struct mrt *mrt, struct rib_entry *re, uint32_t snum)
 		    len) == -1)
 			goto fail;
 
-		DUMP_LONG(hbuf, snum);
-		if (ibuf_add(hbuf, pbuf->buf, ibuf_size(pbuf)) == -1) {
-			log_warn("%s: ibuf_add error", __func__);
+		if (ibuf_add_n32(hbuf, snum) == -1)
+			goto fail;
+		if (ibuf_add_ibuf(hbuf, pbuf) == -1)
+			goto fail;
+		if (ibuf_add_n16(hbuf, nump) == -1)
 			goto fail;
-		}
-		DUMP_SHORT(hbuf, nump);
 
-		ibuf_close(&mrt->wbuf, hbuf);
-		ibuf_close(&mrt->wbuf, nbuf);
+		ibuf_close(mrt->wbuf, hbuf);
+		ibuf_close(mrt->wbuf, nbuf);
 		hbuf = NULL;
 		nbuf = NULL;
 	}
@@ -766,15 +756,15 @@ mrt_dump_entry_v2(struct mrt *mrt, struct rib_entry *re, uint32_t snum)
 		    len) == -1)
 			goto fail;
 
-		DUMP_LONG(hbuf, snum);
-		if (ibuf_add(hbuf, pbuf->buf, ibuf_size(pbuf)) == -1) {
-			log_warn("%s: ibuf_add error", __func__);
+		if (ibuf_add_n32(hbuf, snum) == -1)
+			goto fail;
+		if (ibuf_add_ibuf(hbuf, pbuf) == -1)
+			goto fail;
+		if (ibuf_add_n16(hbuf, apnump) == -1)
 			goto fail;
-		}
-		DUMP_SHORT(hbuf, apnump);
 
-		ibuf_close(&mrt->wbuf, hbuf);
-		ibuf_close(&mrt->wbuf, apbuf);
+		ibuf_close(mrt->wbuf, hbuf);
+		ibuf_close(mrt->wbuf, apbuf);
 		hbuf = NULL;
 		apbuf = NULL;
 	}
@@ -782,6 +772,7 @@ mrt_dump_entry_v2(struct mrt *mrt, struct rib_entry *re, uint32_t snum)
 	ibuf_free(pbuf);
 	return (0);
 fail:
+	log_warn("%s: ibuf error", __func__);
 	ibuf_free(apbuf);
 	ibuf_free(nbuf);
 	ibuf_free(hbuf);
@@ -822,46 +813,45 @@ mrt_dump_v2_hdr(struct mrt *mrt, struct bgpd_config *conf)
 		return (-1);
 	}
 
-	DUMP_NLONG(buf, conf->bgpid);
+	if (ibuf_add_n32(buf, conf->bgpid) == -1)
+		goto fail;
 	nlen = strlen(mrt->rib);
 	if (nlen > 0)
 		nlen += 1;
-	DUMP_SHORT(buf, nlen);
-	if (ibuf_add(buf, mrt->rib, nlen) == -1) {
-		log_warn("%s: ibuf_add error", __func__);
+	if (ibuf_add_n16(buf, nlen) == -1)
+		goto fail;
+	if (ibuf_add(buf, mrt->rib, nlen) == -1)
 		goto fail;
-	}
 
 	off = ibuf_size(buf);
-	if (ibuf_reserve(buf, sizeof(nump)) == NULL) {
-		log_warn("%s: ibuf_reserve error", __func__);
+	if (ibuf_add_zero(buf, sizeof(nump)) == -1)
 		goto fail;
-	}
 	arg.nump = 0;
 	arg.buf = buf;
 	peer_foreach(mrt_dump_v2_hdr_peer, &arg);
 	if (arg.nump == -1)
 		goto fail;
 
-	nump = htons(arg.nump);
-	memcpy(ibuf_seek(buf, off, sizeof(nump)), &nump, sizeof(nump));
+	if (ibuf_set_n16(buf, off, arg.nump) == -1)
+		goto fail;
 
 	len = ibuf_size(buf);
 	if (mrt_dump_hdr_rde(&hbuf, MSG_TABLE_DUMP_V2,
 	    MRT_DUMP_V2_PEER_INDEX_TABLE, len) == -1)
 		goto fail;
 
-	ibuf_close(&mrt->wbuf, hbuf);
-	ibuf_close(&mrt->wbuf, buf);
+	ibuf_close(mrt->wbuf, hbuf);
+	ibuf_close(mrt->wbuf, buf);
 
 	return (0);
 fail:
+	log_warn("%s: ibuf error", __func__);
 	ibuf_free(hbuf);
 	ibuf_free(buf);
 	return (-1);
 }
 
-int
+static int
 mrt_dump_peer(struct ibuf *buf, struct rde_peer *peer)
 {
 	uint8_t	type = 0;
@@ -871,35 +861,41 @@ mrt_dump_peer(struct ibuf *buf, struct rde_peer *peer)
 	if (peer->remote_addr.aid == AID_INET6)
 		type |= MRT_DUMP_V2_PEER_BIT_I;
 
-	DUMP_BYTE(buf, type);
-	DUMP_LONG(buf, peer->remote_bgpid);
+	if (ibuf_add_n8(buf, type) == -1)
+		goto fail;
+	if (ibuf_add_n32(buf, peer->remote_bgpid) == -1)
+		goto fail;
 
 	switch (peer->remote_addr.aid) {
 	case AID_INET:
-		DUMP_NLONG(buf, peer->remote_addr.v4.s_addr);
+		if (ibuf_add(buf, &peer->remote_addr.v4,
+		    sizeof(peer->remote_addr.v4)) == -1)
+			goto fail;
 		break;
 	case AID_INET6:
 		if (ibuf_add(buf, &peer->remote_addr.v6,
-		    sizeof(struct in6_addr)) == -1) {
-			log_warn("mrt_dump_peer: ibuf_add error");
+		    sizeof(peer->remote_addr.v6)) == -1)
 			goto fail;
-		}
 		break;
 	case AID_UNSPEC: /* XXX special handling for peerself? */
-		DUMP_NLONG(buf, 0);
+		if (ibuf_add_n32(buf, 0) == -1)
+			goto fail;
 		break;
 	default:
 		log_warnx("king bula found new AF in %s", __func__);
 		goto fail;
 	}
 
-	if (peer->capa.as4byte)
-		DUMP_LONG(buf, peer->conf.remote_as);
-	else
-		DUMP_SHORT(buf, peer->short_as);
-
+	if (peer->capa.as4byte) {
+		if (ibuf_add_n32(buf, peer->conf.remote_as) == -1)
+			goto fail;
+	} else {
+		if (ibuf_add_n16(buf, peer->short_as) == -1)
+			goto fail;
+	}
 	return (0);
 fail:
+	log_warn("%s: ibuf error", __func__);
 	return (-1);
 }
 
@@ -935,23 +931,24 @@ mrt_done(struct mrt *mrtbuf)
 	mrtbuf->state = MRT_STATE_REMOVE;
 }
 
-int
+static int
 mrt_dump_hdr_se(struct ibuf ** bp, struct peer *peer, uint16_t type,
     uint16_t subtype, uint32_t len, int swap)
 {
 	struct timespec	time;
 
 	if ((*bp = ibuf_dynamic(MRT_ET_HEADER_SIZE, MRT_ET_HEADER_SIZE +
-	    MRT_BGP4MP_AS4_IPv6_HEADER_SIZE + len)) == NULL) {
-		log_warn("mrt_dump_hdr_se: ibuf_dynamic error");
+	    MRT_BGP4MP_AS4_IPv6_HEADER_SIZE + len)) == NULL)
 		return (-1);
-	}
 
 	clock_gettime(CLOCK_REALTIME, &time);
 
-	DUMP_LONG(*bp, time.tv_sec);
-	DUMP_SHORT(*bp, type);
-	DUMP_SHORT(*bp, subtype);
+	if (ibuf_add_n32(*bp, time.tv_sec) == -1)
+		goto fail;
+	if (ibuf_add_n16(*bp, type) == -1)
+		goto fail;
+	if (ibuf_add_n16(*bp, subtype) == -1)
+		goto fail;
 
 	switch (peer->local.aid) {
 	case AID_INET:
@@ -977,56 +974,67 @@ mrt_dump_hdr_se(struct ibuf ** bp, struct peer *peer, uint16_t type,
 		goto fail;
 	}
 
-	DUMP_LONG(*bp, len);
+	if (ibuf_add_n32(*bp, len) == -1)
+		goto fail;
 	/* millisecond field use by the _ET format */
-	DUMP_LONG(*bp, time.tv_nsec / 1000);
+	if (ibuf_add_n32(*bp, time.tv_nsec / 1000) == -1)
+		goto fail;
 
 	if (subtype == BGP4MP_STATE_CHANGE_AS4 ||
 	    subtype == BGP4MP_MESSAGE_AS4 ||
 	    subtype == BGP4MP_MESSAGE_AS4_ADDPATH) {
 		if (!swap)
-			DUMP_LONG(*bp, peer->conf.local_as);
-		DUMP_LONG(*bp, peer->conf.remote_as);
+			if (ibuf_add_n32(*bp, peer->conf.local_as) == -1)
+				goto fail;
+		if (ibuf_add_n32(*bp, peer->conf.remote_as) == -1)
+			goto fail;
 		if (swap)
-			DUMP_LONG(*bp, peer->conf.local_as);
+			if (ibuf_add_n32(*bp, peer->conf.local_as) == -1)
+				goto fail;
 	} else {
 		if (!swap)
-			DUMP_SHORT(*bp, peer->conf.local_short_as);
-		DUMP_SHORT(*bp, peer->short_as);
+			if (ibuf_add_n16(*bp, peer->conf.local_short_as) == -1)
+				goto fail;
+		if (ibuf_add_n16(*bp, peer->short_as) == -1)
+			goto fail;
 		if (swap)
-			DUMP_SHORT(*bp, peer->conf.local_short_as);
+			if (ibuf_add_n16(*bp, peer->conf.local_short_as) == -1)
+				goto fail;
 	}
 
-	DUMP_SHORT(*bp, /* ifindex */ 0);
+	if (ibuf_add_n16(*bp, /* ifindex */ 0) == -1)
+		goto fail;
 
 	switch (peer->local.aid) {
 	case AID_INET:
-		DUMP_SHORT(*bp, AFI_IPv4);
+		if (ibuf_add_n16(*bp, AFI_IPv4) == -1)
+			goto fail;
 		if (!swap)
-			DUMP_NLONG(*bp, peer->local.v4.s_addr);
-		DUMP_NLONG(*bp, peer->remote.v4.s_addr);
+			if (ibuf_add(*bp, &peer->local.v4,
+			    sizeof(peer->local.v4)) == -1)
+				goto fail;
+		if (ibuf_add(*bp, &peer->remote.v4,
+		    sizeof(peer->remote.v4)) == -1)
+			goto fail;
 		if (swap)
-			DUMP_NLONG(*bp, peer->local.v4.s_addr);
+			if (ibuf_add(*bp, &peer->local.v4,
+			    sizeof(peer->local.v4)) == -1)
+				goto fail;
 		break;
 	case AID_INET6:
-		DUMP_SHORT(*bp, AFI_IPv6);
+		if (ibuf_add_n16(*bp, AFI_IPv6) == -1)
+			goto fail;
 		if (!swap)
 			if (ibuf_add(*bp, &peer->local.v6,
-			    sizeof(struct in6_addr)) == -1) {
-				log_warn("mrt_dump_hdr_se: ibuf_add error");
+			    sizeof(peer->local.v6)) == -1)
 				goto fail;
-			}
 		if (ibuf_add(*bp, &peer->remote.v6,
-		    sizeof(struct in6_addr)) == -1) {
-			log_warn("mrt_dump_hdr_se: ibuf_add error");
+		    sizeof(peer->remote.v6)) == -1)
 			goto fail;
-		}
 		if (swap)
 			if (ibuf_add(*bp, &peer->local.v6,
-			    sizeof(struct in6_addr)) == -1) {
-				log_warn("mrt_dump_hdr_se: ibuf_add error");
+			    sizeof(peer->local.v6)) == -1)
 				goto fail;
-			}
 		break;
 	}
 
@@ -1034,6 +1042,7 @@ mrt_dump_hdr_se(struct ibuf ** bp, struct peer *peer, uint16_t type,
 
 fail:
 	ibuf_free(*bp);
+	*bp = NULL;
 	return (-1);
 }
 
@@ -1045,16 +1054,17 @@ mrt_dump_hdr_rde(struct ibuf **bp, uint16_t type, uint16_t subtype,
 
 	if ((*bp = ibuf_dynamic(MRT_HEADER_SIZE, MRT_HEADER_SIZE +
 	    MRT_BGP4MP_AS4_IPv6_HEADER_SIZE + MRT_BGP4MP_IPv6_ENTRY_SIZE)) ==
-	    NULL) {
-		log_warn("mrt_dump_hdr_rde: ibuf_dynamic error");
+	    NULL)
 		return (-1);
-	}
 
 	clock_gettime(CLOCK_REALTIME, &time);
 
-	DUMP_LONG(*bp, time.tv_sec);
-	DUMP_SHORT(*bp, type);
-	DUMP_SHORT(*bp, subtype);
+	if (ibuf_add_n32(*bp, time.tv_sec) == -1)
+		goto fail;
+	if (ibuf_add_n16(*bp, type) == -1)
+		goto fail;
+	if (ibuf_add_n16(*bp, subtype) == -1)
+		goto fail;
 
 	switch (type) {
 	case MSG_TABLE_DUMP:
@@ -1066,11 +1076,13 @@ mrt_dump_hdr_rde(struct ibuf **bp, uint16_t type, uint16_t subtype,
 			len += MRT_DUMP_HEADER_SIZE_V6;
 			break;
 		}
-		DUMP_LONG(*bp, len);
+		if (ibuf_add_n32(*bp, len) == -1)
+			goto fail;
 		break;
 	case MSG_PROTOCOL_BGP4MP:
 	case MSG_TABLE_DUMP_V2:
-		DUMP_LONG(*bp, len);
+		if (ibuf_add_n32(*bp, len) == -1)
+			goto fail;
 		break;
 	default:
 		log_warnx("mrt_dump_hdr_rde: unsupported type");
@@ -1087,9 +1099,7 @@ fail:
 void
 mrt_write(struct mrt *mrt)
 {
-	int	r;
-
-	if ((r = ibuf_write(&mrt->wbuf)) == -1 && errno != EAGAIN) {
+	if (ibuf_write(mrt->fd, mrt->wbuf) == -1) {
 		log_warn("mrt dump aborted, mrt_write");
 		mrt_clean(mrt);
 		mrt_done(mrt);
@@ -1099,14 +1109,9 @@ mrt_write(struct mrt *mrt)
 void
 mrt_clean(struct mrt *mrt)
 {
-	struct ibuf	*b;
-
-	close(mrt->wbuf.fd);
-	while ((b = TAILQ_FIRST(&mrt->wbuf.bufs))) {
-		TAILQ_REMOVE(&mrt->wbuf.bufs, b, entry);
-		ibuf_free(b);
-	}
-	mrt->wbuf.queued = 0;
+	close(mrt->fd);
+	msgbuf_free(mrt->wbuf);
+	mrt->wbuf = NULL;
 }
 
 static struct imsgbuf	*mrt_imsgbuf[2];
