@@ -1,4 +1,4 @@
-/*	$OpenBSD: rde_rib.c,v 1.262 2024/05/29 10:34:56 claudio Exp $ */
+/*	$OpenBSD: rde_rib.c,v 1.266 2024/12/12 20:19:03 claudio Exp $ */
 
 /*
  * Copyright (c) 2003, 2004 Claudio Jeker <claudio@openbsd.org>
@@ -801,10 +801,10 @@ path_put(struct rde_aspath *asp)
 static int	prefix_add(struct bgpd_addr *, int, struct rib *,
 		    struct rde_peer *, uint32_t, uint32_t, struct rde_aspath *,
 		    struct rde_community *, struct nexthop *,
-		    uint8_t, uint8_t);
+		    uint8_t, uint8_t, int);
 static int	prefix_move(struct prefix *, struct rde_peer *,
 		    struct rde_aspath *, struct rde_community *,
-		    struct nexthop *, uint8_t, uint8_t);
+		    struct nexthop *, uint8_t, uint8_t, int);
 
 static void	prefix_link(struct prefix *, struct rib_entry *,
 		    struct pt_entry *, struct rde_peer *, uint32_t, uint32_t,
@@ -967,8 +967,8 @@ prefix_adjout_match(struct rde_peer *peer, struct bgpd_addr *addr)
  */
 int
 prefix_update(struct rib *rib, struct rde_peer *peer, uint32_t path_id,
-    uint32_t path_id_tx, struct filterstate *state, struct bgpd_addr *prefix,
-    int prefixlen)
+    uint32_t path_id_tx, struct filterstate *state, int filtered,
+    struct bgpd_addr *prefix, int prefixlen)
 {
 	struct rde_aspath	*asp, *nasp = &state->aspath;
 	struct rde_community	*comm, *ncomm = &state->communities;
@@ -987,6 +987,10 @@ prefix_update(struct rib *rib, struct rde_peer *peer, uint32_t path_id,
 			/* no change, update last change */
 			p->lastchange = getmonotime();
 			p->validation_state = state->vstate;
+			if (filtered)
+				p->flags |= PREFIX_FLAG_FILTERED;
+			else
+				p->flags &= ~PREFIX_FLAG_FILTERED;
 			return (0);
 		}
 	}
@@ -1010,11 +1014,11 @@ prefix_update(struct rib *rib, struct rde_peer *peer, uint32_t path_id,
 	/* If the prefix was found move it else add it to the RIB. */
 	if (p != NULL)
 		return (prefix_move(p, peer, asp, comm, state->nexthop,
-		    state->nhflags, state->vstate));
+		    state->nhflags, state->vstate, filtered));
 	else
 		return (prefix_add(prefix, prefixlen, rib, peer, path_id,
 		    path_id_tx, asp, comm, state->nexthop, state->nhflags,
-		    state->vstate));
+		    state->vstate, filtered));
 }
 
 /*
@@ -1024,7 +1028,7 @@ static int
 prefix_add(struct bgpd_addr *prefix, int prefixlen, struct rib *rib,
     struct rde_peer *peer, uint32_t path_id, uint32_t path_id_tx,
     struct rde_aspath *asp, struct rde_community *comm,
-    struct nexthop *nexthop, uint8_t nhflags, uint8_t vstate)
+    struct nexthop *nexthop, uint8_t nhflags, uint8_t vstate, int filtered)
 {
 	struct pt_entry	*pte;
 	struct prefix		*p;
@@ -1041,6 +1045,9 @@ prefix_add(struct bgpd_addr *prefix, int prefixlen, struct rib *rib,
 	prefix_link(p, re, re->prefix, peer, path_id, path_id_tx, asp, comm,
 	    nexthop, nhflags, vstate);
 
+	if (filtered)
+		p->flags |= PREFIX_FLAG_FILTERED;
+
 	/* add possible pftable reference form aspath */
 	if (asp && asp->pftableid)
 		rde_pftable_add(asp->pftableid, p);
@@ -1055,7 +1062,7 @@ prefix_add(struct bgpd_addr *prefix, int prefixlen, struct rib *rib,
 static int
 prefix_move(struct prefix *p, struct rde_peer *peer,
     struct rde_aspath *asp, struct rde_community *comm,
-    struct nexthop *nexthop, uint8_t nhflags, uint8_t vstate)
+    struct nexthop *nexthop, uint8_t nhflags, uint8_t vstate, int filtered)
 {
 	struct prefix		*np;
 
@@ -1070,6 +1077,9 @@ prefix_move(struct prefix *p, struct rde_peer *peer,
 	prefix_link(np, prefix_re(p), p->pt, peer, p->path_id, p->path_id_tx,
 	    asp, comm, nexthop, nhflags, vstate);
 
+	if (filtered)
+		np->flags |= PREFIX_FLAG_FILTERED;
+
 	/* add possible pftable reference from new aspath */
 	if (asp && asp->pftableid)
 		rde_pftable_add(asp->pftableid, np);
@@ -1263,7 +1273,6 @@ prefix_adjout_update(struct prefix *p, struct rde_peer *peer,
 			/* nothing changed */
 			p->validation_state = state->vstate;
 			p->lastchange = getmonotime();
-			p->flags &= ~PREFIX_FLAG_STALE;
 			return;
 		}
 
@@ -1311,10 +1320,12 @@ prefix_adjout_update(struct prefix *p, struct rde_peer *peer,
 
 	if (p->flags & PREFIX_FLAG_MASK)
 		fatalx("%s: bad flags %x", __func__, p->flags);
-	p->flags |= PREFIX_FLAG_UPDATE;
-	if (RB_INSERT(prefix_tree, &peer->updates[pte->aid], p) != NULL)
-		fatalx("%s: RB tree invariant violated", __func__);
-	peer->stats.pending_update++;
+	if (peer_is_up(peer)) {
+		p->flags |= PREFIX_FLAG_UPDATE;
+		if (RB_INSERT(prefix_tree, &peer->updates[pte->aid], p) != NULL)
+			fatalx("%s: RB tree invariant violated", __func__);
+		peer->stats.pending_update++;
+	}
 }
 
 /*
@@ -1332,7 +1343,6 @@ prefix_adjout_withdraw(struct prefix *p)
 	/* already a withdraw, shortcut */
 	if (p->flags & PREFIX_FLAG_WITHDRAW) {
 		p->lastchange = getmonotime();
-		p->flags &= ~PREFIX_FLAG_STALE;
 		return;
 	}
 	/* pending update just got withdrawn */
@@ -1350,10 +1360,17 @@ prefix_adjout_withdraw(struct prefix *p)
 	p->flags &= ~PREFIX_FLAG_MASK;
 	p->lastchange = getmonotime();
 
-	p->flags |= PREFIX_FLAG_WITHDRAW;
-	if (RB_INSERT(prefix_tree, &peer->withdraws[p->pt->aid], p) != NULL)
-		fatalx("%s: RB tree invariant violated", __func__);
-	peer->stats.pending_withdraw++;
+	if (peer_is_up(peer)) {
+		p->flags |= PREFIX_FLAG_WITHDRAW;
+		if (RB_INSERT(prefix_tree, &peer->withdraws[p->pt->aid],
+		    p) != NULL)
+			fatalx("%s: RB tree invariant violated", __func__);
+		peer->stats.pending_withdraw++;
+	} else {
+		/* mark prefix dead to skip unlink on destroy */
+		p->flags |= PREFIX_FLAG_DEAD;
+		prefix_adjout_destroy(p);
+	}
 }
 
 void
@@ -1398,6 +1415,38 @@ prefix_adjout_destroy(struct prefix *p)
 	}
 }
 
+void
+prefix_adjout_flush_pending(struct rde_peer *peer)
+{
+	struct prefix *p, *np;
+	uint8_t aid;
+
+	for (aid = AID_MIN; aid < AID_MAX; aid++) {
+		RB_FOREACH_SAFE(p, prefix_tree, &peer->withdraws[aid], np) {
+			prefix_adjout_destroy(p);
+		}
+		RB_FOREACH_SAFE(p, prefix_tree, &peer->updates[aid], np) {
+			p->flags &= ~PREFIX_FLAG_UPDATE;
+			RB_REMOVE(prefix_tree, &peer->updates[aid], p);
+			peer->stats.pending_update--;
+		}
+	}
+}
+
+int
+prefix_adjout_reaper(struct rde_peer *peer)
+{
+	struct prefix *p, *np;
+	int count = RDE_REAPER_ROUNDS;
+
+	RB_FOREACH_SAFE(p, prefix_index, &peer->adj_rib_out, np) {
+		prefix_adjout_destroy(p);
+		if (count-- <= 0)
+			return 0;
+	}
+	return 1;
+}
+
 static struct prefix *
 prefix_restart(struct rib_context *ctx)
 {
