@@ -1,4 +1,4 @@
-/*	$OpenBSD: rtr_proto.c,v 1.40 2024/09/10 08:41:13 claudio Exp $ */
+/*	$OpenBSD: rtr_proto.c,v 1.47 2024/11/26 13:59:54 claudio Exp $ */
 
 /*
  * Copyright (c) 2020 Claudio Jeker <claudio@openbsd.org>
@@ -189,9 +189,8 @@ struct rtr_session {
 	char				descr[PEER_DESCR_LEN];
 	struct roa_tree			roa_set;
 	struct aspa_tree		aspa;
-	struct ibuf_read		r;
-	struct msgbuf			w;
 	struct timer_head		timers;
+	struct msgbuf			*w;
 	uint32_t			id;		/* rtr_config id */
 	uint32_t			serial;
 	uint32_t			refresh;
@@ -360,7 +359,7 @@ rtr_send_error(struct rtr_session *rs, struct ibuf *pdu, enum rtr_error err,
 		goto fail;
 	if (ibuf_add(buf, rs->last_sent_msg, mlen) == -1)
 		goto fail;
-	ibuf_close(&rs->w, buf);
+	ibuf_close(rs->w, buf);
 
 	rtr_fsm(rs, RTR_EVNT_SEND_ERROR);
 	return;
@@ -378,7 +377,7 @@ rtr_send_reset_query(struct rtr_session *rs)
 	buf = rtr_newmsg(rs, RESET_QUERY, 0, 0);
 	if (buf == NULL)
 		goto fail;
-	ibuf_close(&rs->w, buf);
+	ibuf_close(rs->w, buf);
 	return;
 
  fail:
@@ -397,7 +396,7 @@ rtr_send_serial_query(struct rtr_session *rs)
 		goto fail;
 	if (ibuf_add_n32(buf, rs->serial) == -1)
 		goto fail;
-	ibuf_close(&rs->w, buf);
+	ibuf_close(rs->w, buf);
 	return;
 
  fail:
@@ -424,30 +423,54 @@ rtr_check_session_id(struct rtr_session *rs, uint16_t session_id,
 }
 
 /*
- * Parse the common rtr header (first 8 bytes) including the
- * included length field.
- * Returns -1 on failure. On success msgtype and msglen are set
- * and the function return 0.
+ * Callback for ibuf_read to get the size of a PDU.
  */
-static int
-rtr_parse_header(struct rtr_session *rs, struct ibuf *hdr,
-    size_t *msglen, enum rtr_pdu_type *msgtype)
+static struct ibuf *
+rtr_reader_callback(struct ibuf *hdr, void *arg, int *fd)
 {
+	struct rtr_session *rs = arg;
 	struct rtr_header rh;
-	size_t len;
-	uint16_t errcode;
+	struct ibuf *b;
+	ssize_t len;
 
 	if (ibuf_get(hdr, &rh, sizeof(rh)) == -1)
-		fatal("%s: ibuf_get", __func__);
+		return NULL;
 
 	len = ntohl(rh.length);
 
 	if (len > RTR_MAX_PDU_SIZE) {
 		rtr_send_error(rs, hdr, CORRUPT_DATA, "%s: too big: %zu bytes",
 		    log_rtr_type(rh.type), len);
-		return -1;
+		errno = ERANGE;
+		return NULL;
 	}
 
+	if ((b = ibuf_open(len)) == NULL)
+		return NULL;
+	return b;
+}
+
+/*
+ * Parse the common rtr header (first 8 bytes) including the
+ * included length field.
+ * Returns -1 on failure. On success msgtype and msglen are set
+ * and the function return 0.
+ */
+static int
+rtr_parse_header(struct rtr_session *rs, struct ibuf *msg,
+    enum rtr_pdu_type *msgtype)
+{
+	struct rtr_header rh;
+	struct ibuf hdr;
+	size_t len;
+	uint16_t errcode;
+
+	len = ibuf_size(msg);
+
+	ibuf_from_ibuf(&hdr, msg);
+	if (ibuf_get(&hdr, &rh, sizeof(rh)) == -1)
+		fatal("%s: ibuf_get", __func__);
+
 	if (rs->state == RTR_STATE_NEGOTIATION) {
 		switch (rh.type) {
 		case CACHE_RESPONSE:
@@ -473,7 +496,7 @@ rtr_parse_header(struct rtr_session *rs, struct ibuf *hdr,
 			/* ignore SERIAL_NOTIFY */
 			break;
 		default:
-			rtr_send_error(rs, hdr, CORRUPT_DATA,
+			rtr_send_error(rs, msg, CORRUPT_DATA,
 			    "%s: out of context", log_rtr_type(rh.type));
 			return -1;
 		}
@@ -528,23 +551,22 @@ rtr_parse_header(struct rtr_session *rs, struct ibuf *hdr,
 			goto badlen;
 		break;
 	default:
-		rtr_send_error(rs, hdr, UNSUPP_PDU_TYPE, "type %s",
+		rtr_send_error(rs, msg, UNSUPP_PDU_TYPE, "type %s",
 		    log_rtr_type(rh.type));
 		return -1;
 	}
 
-	*msglen = len;
 	*msgtype = rh.type;
 
 	return 0;
 
  badlen:
-	rtr_send_error(rs, hdr, CORRUPT_DATA, "%s: bad length: %zu bytes",
+	rtr_send_error(rs, msg, CORRUPT_DATA, "%s: bad length: %zu bytes",
 	    log_rtr_type(rh.type), len);
 	return -1;
 
  badversion:
-	rtr_send_error(rs, hdr, UNEXP_PROTOCOL_VERS, "%s: version %d",
+	rtr_send_error(rs, msg, UNEXP_PROTOCOL_VERS, "%s: version %d",
 	    log_rtr_type(rh.type), rh.version);
 	return -1;
 }
@@ -1019,76 +1041,58 @@ rtr_parse_error(struct rtr_session *rs, struct ibuf *pdu)
  * a retry will be done.
  */
 static void
-rtr_process_msg(struct rtr_session *rs)
+rtr_process_msg(struct rtr_session *rs, struct ibuf *msg)
 {
-	struct ibuf rbuf, hdr, msg;
-	size_t msglen;
 	enum rtr_pdu_type msgtype;
 
-	ibuf_from_buffer(&rbuf, rs->r.buf, rs->r.wpos);
+	/* parse and check header */
+	if (rtr_parse_header(rs, msg, &msgtype) == -1)
+		return;
 
-	for (;;) {
-		if (ibuf_size(&rbuf) < sizeof(struct rtr_header))
-			break;
-
-		/* parse header */
-		ibuf_from_buffer(&hdr, ibuf_data(&rbuf),
-		    sizeof(struct rtr_header));
-		if (rtr_parse_header(rs, &hdr, &msglen, &msgtype) == -1)
+	switch (msgtype) {
+	case SERIAL_NOTIFY:
+		if (rtr_parse_notify(rs, msg) == -1)
 			return;
-
-		/* extract message */
-		if (ibuf_get_ibuf(&rbuf, msglen, &msg) == -1)
-			break;
-
-		switch (msgtype) {
-		case SERIAL_NOTIFY:
-			if (rtr_parse_notify(rs, &msg) == -1)
-				return;
-			break;
-		case CACHE_RESPONSE:
-			if (rtr_parse_cache_response(rs, &msg) == -1)
-				return;
-			break;
-		case IPV4_PREFIX:
-			if (rtr_parse_ipv4_prefix(rs, &msg) == -1)
-				return;
-			break;
-		case IPV6_PREFIX:
-			if (rtr_parse_ipv6_prefix(rs, &msg) == -1)
-				return;
-			break;
-		case END_OF_DATA:
-			if (rtr_parse_end_of_data(rs, &msg) == -1)
-				return;
-			break;
-		case CACHE_RESET:
-			if (rtr_parse_cache_reset(rs, &msg) == -1)
-				return;
-			break;
-		case ROUTER_KEY:
-			/* silently ignore router key */
-			break;
-		case ERROR_REPORT:
-			if (rtr_parse_error(rs, &msg) == -1) {
-				/* no need to send back an error */
-				return;
-			}
-			break;
-		case ASPA:
-			if (rtr_parse_aspa(rs, &msg) == -1)
-				return;
-			break;
-		default:
-			/* unreachable, checked in rtr_parse_header() */
-			rtr_send_error(rs, &msg, UNSUPP_PDU_TYPE, "type %s",
-			    log_rtr_type(msgtype));
+		break;
+	case CACHE_RESPONSE:
+		if (rtr_parse_cache_response(rs, msg) == -1)
+			return;
+		break;
+	case IPV4_PREFIX:
+		if (rtr_parse_ipv4_prefix(rs, msg) == -1)
+			return;
+		break;
+	case IPV6_PREFIX:
+		if (rtr_parse_ipv6_prefix(rs, msg) == -1)
+			return;
+		break;
+	case END_OF_DATA:
+		if (rtr_parse_end_of_data(rs, msg) == -1)
+			return;
+		break;
+	case CACHE_RESET:
+		if (rtr_parse_cache_reset(rs, msg) == -1)
+			return;
+		break;
+	case ROUTER_KEY:
+		/* silently ignore router key */
+		break;
+	case ERROR_REPORT:
+		if (rtr_parse_error(rs, msg) == -1) {
+			/* no need to send back an error */
 			return;
 		}
+		break;
+	case ASPA:
+		if (rtr_parse_aspa(rs, msg) == -1)
+			return;
+		break;
+	default:
+		/* unreachable, checked in rtr_parse_header() */
+		rtr_send_error(rs, msg, UNSUPP_PDU_TYPE, "type %s",
+		    log_rtr_type(msgtype));
+		return;
 	}
-
-	memmove(&rs->r.buf, ibuf_data(&rbuf), ibuf_size(&rbuf));
-	rs->r.wpos = ibuf_size(&rbuf);
 }
 
 /*
@@ -1126,10 +1130,11 @@ rtr_fsm(struct rtr_session *rs, enum rtr_event event)
 	case RTR_EVNT_CON_CLOSE:
 		if (rs->fd != -1) {
 			/* flush buffers */
-			msgbuf_clear(&rs->w);
-			rs->r.wpos = 0;
+			msgbuf_clear(rs->w);
 			close(rs->fd);
 			rs->fd = -1;
+			rtr_imsg_compose(IMSG_SOCKET_TEARDOWN, rs->id, 0,
+			    NULL, 0);
 		}
 		/* try to reopen session */
 		if (!rs->errored)
@@ -1158,7 +1163,7 @@ rtr_fsm(struct rtr_session *rs, enum rtr_event event)
 		case RTR_STATE_CLOSED:
 		case RTR_STATE_NEGOTIATION:
 			timer_set(&rs->timers, Timer_Rtr_Retry, rs->retry);
-			rtr_imsg_compose(IMSG_SOCKET_CONN, rs->id, 0, NULL, 0);
+			rtr_imsg_compose(IMSG_SOCKET_SETUP, rs->id, 0, NULL, 0);
 			break;
 		case RTR_STATE_ESTABLISHED:
 			if (rs->session_id == -1)
@@ -1238,8 +1243,6 @@ rtr_fsm(struct rtr_session *rs, enum rtr_event event)
 		rtr_reset_cache(rs);
 		rtr_recalc();
 		rs->state = RTR_STATE_ERROR;
-		/* flush receive buffer */
-		rs->r.wpos = 0;
 		break;
 	case RTR_EVNT_NEGOTIATION_DONE:
 		rs->state = RTR_STATE_ESTABLISHED;
@@ -1257,8 +1260,7 @@ rtr_fsm(struct rtr_session *rs, enum rtr_event event)
 static void
 rtr_dispatch_msg(struct pollfd *pfd, struct rtr_session *rs)
 {
-	ssize_t n;
-	int error;
+	struct ibuf *b;
 
 	if (pfd->revents & POLLHUP) {
 		log_warnx("rtr %s: Connection closed, hangup", log_rtr(rs));
@@ -1270,38 +1272,35 @@ rtr_dispatch_msg(struct pollfd *pfd, struct rtr_session *rs)
 		rtr_fsm(rs, RTR_EVNT_CON_CLOSE);
 		return;
 	}
-	if (pfd->revents & POLLOUT && msgbuf_queuelen(&rs->w) > 0) {
-		if ((error = ibuf_write(&rs->w)) == -1) {
-			if (errno != EAGAIN) {
-				log_warn("rtr %s: write error", log_rtr(rs));
-				rtr_fsm(rs, RTR_EVNT_CON_CLOSE);
-			}
-		}
-		if (error == 0)
+	if (pfd->revents & POLLOUT && msgbuf_queuelen(rs->w) > 0) {
+		if (ibuf_write(rs->fd, rs->w) == -1) {
+			log_warn("rtr %s: write error", log_rtr(rs));
 			rtr_fsm(rs, RTR_EVNT_CON_CLOSE);
+			return;
+		}
 		if (rs->state == RTR_STATE_ERROR &&
-		    msgbuf_queuelen(&rs->w) == 0)
+		    msgbuf_queuelen(rs->w) == 0)
 			rtr_fsm(rs, RTR_EVNT_CON_CLOSE);
 	}
 	if (pfd->revents & POLLIN) {
-		if ((n = read(rs->fd, rs->r.buf + rs->r.wpos,
-		    sizeof(rs->r.buf) - rs->r.wpos)) == -1) {
-			if (errno != EINTR && errno != EAGAIN) {
-				log_warn("rtr %s: read error", log_rtr(rs));
-				rtr_fsm(rs, RTR_EVNT_CON_CLOSE);
-			}
+		switch (ibuf_read(rs->fd, rs->w)) {
+		case -1:
+			/* if already in error state, ignore */
+			if (rs->state == RTR_STATE_ERROR)
+				return;
+			log_warn("rtr %s: read error", log_rtr(rs));
+			rtr_fsm(rs, RTR_EVNT_CON_CLOSE);
 			return;
-		}
-		if (n == 0) {
+		case 0:
 			rtr_fsm(rs, RTR_EVNT_CON_CLOSE);
 			return;
 		}
-		rs->r.wpos += n;
-
 		/* new data arrived, try to process it */
-		rtr_process_msg(rs);
+		while ((b = msgbuf_get(rs->w)) != NULL) {
+			rtr_process_msg(rs, b);
+			ibuf_free(b);
+		}
 	}
-
 }
 
 void
@@ -1386,7 +1385,7 @@ rtr_poll_events(struct pollfd *pfds, size_t npfds, time_t *timeout)
 		pfd->fd = rs->fd;
 		pfd->events = 0;
 
-		if (msgbuf_queuelen(&rs->w) > 0)
+		if (msgbuf_queuelen(rs->w) > 0)
 			pfd->events |= POLLOUT;
 		if (rs->state >= RTR_STATE_ESTABLISHED)
 			pfd->events |= POLLIN;
@@ -1402,11 +1401,13 @@ rtr_new(uint32_t id, struct rtr_config_msg *conf)
 
 	if ((rs = calloc(1, sizeof(*rs))) == NULL)
 		fatal("RTR session %s", conf->descr);
+	if ((rs->w = msgbuf_new_reader(sizeof(struct rtr_header),
+	    rtr_reader_callback, rs)) == NULL)
+		fatal("RTR session %s", conf->descr);
 
 	RB_INIT(&rs->roa_set);
 	RB_INIT(&rs->aspa);
 	TAILQ_INIT(&rs->timers);
-	msgbuf_init(&rs->w);
 
 	strlcpy(rs->descr, conf->descr, sizeof(rs->descr));
 	rs->id = id;
@@ -1453,6 +1454,7 @@ rtr_free(struct rtr_session *rs)
 	rtr_reset_cache(rs);
 	rtr_fsm(rs, RTR_EVNT_CON_CLOSE);
 	timer_remove_all(&rs->timers);
+	msgbuf_free(rs->w);
 	free(rs);
 }
 
@@ -1470,7 +1472,7 @@ rtr_open(struct rtr_session *rs, int fd)
 		rs->prev_version = rtr_max_session_version(rs);
 	}
 
-	rs->fd = rs->w.fd = fd;
+	rs->fd = fd;
 	rtr_fsm(rs, RTR_EVNT_CON_OPEN);
 }
 
