@@ -1,4 +1,4 @@
-/*	$OpenBSD: session.h,v 1.170 2024/05/18 11:17:30 jsg Exp $ */
+/*	$OpenBSD: session.h,v 1.184 2024/12/16 16:10:10 claudio Exp $ */
 
 /*
  * Copyright (c) 2003, 2004 Henning Brauer <henning@openbsd.org>
@@ -25,8 +25,9 @@
 #define	INTERVAL_HOLD_INITIAL		240
 #define	INTERVAL_HOLD			90
 #define	INTERVAL_IDLE_HOLD_INITIAL	30
-#define	INTERVAL_HOLD_CLONED		3600
 #define	INTERVAL_HOLD_DEMOTED		60
+#define	INTERVAL_STALE			180
+#define	INTERVAL_SESSION_DOWN		3600
 #define	MAX_IDLE_HOLD			3600
 #define	MSGSIZE_HEADER			19
 #define	MSGSIZE_HEADER_MARKER		16
@@ -64,15 +65,16 @@ enum session_events {
 	EVNT_RCVD_OPEN,
 	EVNT_RCVD_KEEPALIVE,
 	EVNT_RCVD_UPDATE,
-	EVNT_RCVD_NOTIFICATION
+	EVNT_RCVD_NOTIFICATION,
+	EVNT_RCVD_GRACE_NOTIFICATION,
 };
 
 enum msg_type {
-	OPEN = 1,
-	UPDATE,
-	NOTIFICATION,
-	KEEPALIVE,
-	RREFRESH
+	MSG_OPEN = 1,
+	MSG_UPDATE,
+	MSG_NOTIFICATION,
+	MSG_KEEPALIVE,
+	MSG_RREFRESH
 };
 
 enum suberr_header {
@@ -106,12 +108,6 @@ enum opt_params {
 	OPT_PARAM_EXT_LEN=255,
 };
 
-struct bgp_msg {
-	struct ibuf	*buf;
-	enum msg_type	 type;
-	uint16_t	 len;
-};
-
 struct bgpd_sysdep {
 	uint8_t			no_pfkey;
 	uint8_t			no_md5sig;
@@ -151,6 +147,7 @@ struct peer_stats {
 	time_t			 last_updown;
 	time_t			 last_read;
 	time_t			 last_write;
+	uint32_t		 msg_queue_len;
 	uint32_t		 prefix_cnt;
 	uint32_t		 prefix_out_cnt;
 	uint32_t		 pending_update;
@@ -162,6 +159,15 @@ struct peer_stats {
 	char			 last_reason[REASON_LEN];
 };
 
+struct auth_state {
+	struct bgpd_addr	local_addr;
+	struct bgpd_addr	remote_addr;
+	uint32_t		spi_in;
+	uint32_t		spi_out;
+	enum auth_method	method;
+	uint8_t			established;
+};
+
 enum Timer {
 	Timer_None,
 	Timer_ConnectRetry,
@@ -172,6 +178,7 @@ enum Timer {
 	Timer_IdleHoldReset,
 	Timer_CarpUndemote,
 	Timer_RestartTimeout,
+	Timer_SessionDown,
 	Timer_Rtr_Refresh,
 	Timer_Rtr_Retry,
 	Timer_Rtr_Expire,
@@ -196,19 +203,13 @@ struct peer {
 		struct capabilities	peer;
 		struct capabilities	neg;
 	}			 capa;
-	struct {
-		struct bgpd_addr	local_addr;
-		uint32_t		spi_in;
-		uint32_t		spi_out;
-		enum auth_method	method;
-		uint8_t			established;
-	}			 auth;
+	struct auth_state	 auth_state;
+	struct auth_config	 auth_conf;
 	struct bgpd_addr	 local;
 	struct bgpd_addr	 local_alt;
 	struct bgpd_addr	 remote;
 	struct timer_head	 timers;
-	struct msgbuf		 wbuf;
-	struct ibuf_read	*rbuf;
+	struct msgbuf		*wbuf;
 	struct peer		*template;
 	int			 fd;
 	int			 lasterr;
@@ -229,6 +230,7 @@ struct peer {
 	uint8_t			 passive;
 	uint8_t			 throttled;
 	uint8_t			 rpending;
+	uint8_t			 rdesession;
 };
 
 extern time_t		 pauseaccept;
@@ -246,6 +248,7 @@ int	 carp_demote_set(char *, int);
 
 /* config.c */
 void	 merge_config(struct bgpd_config *, struct bgpd_config *);
+void	 free_deleted_peers(struct bgpd_config *);
 int	 prepare_listeners(struct bgpd_config *);
 
 /* control.c */
@@ -267,8 +270,8 @@ void	 log_conn_attempt(const struct peer *, struct sockaddr *,
 	    socklen_t);
 
 /* mrt.c */
-void	 mrt_dump_bgp_msg(struct mrt *, void *, uint16_t,
-	    struct peer *, enum msg_type);
+void	 mrt_dump_bgp_msg(struct mrt *, struct ibuf *, struct peer *,
+	    enum msg_type);
 void	 mrt_dump_state(struct mrt *, uint16_t, uint16_t,
 	    struct peer *);
 void	 mrt_done(struct mrt *);
@@ -276,11 +279,14 @@ void	 mrt_done(struct mrt *);
 /* pfkey.c */
 struct sadb_msg;
 int	pfkey_read(int, struct sadb_msg *);
-int	pfkey_establish(struct peer *);
-int	pfkey_remove(struct peer *);
+int	pfkey_establish(struct auth_state *, struct auth_config *,
+	    const struct bgpd_addr *, const struct bgpd_addr *);
+int	pfkey_remove(struct auth_state *);
 int	pfkey_init(void);
-int	tcp_md5_check(int, struct peer *);
-int	tcp_md5_set(int, struct peer *);
+int	pfkey_send_conf(struct imsgbuf *, uint32_t, struct auth_config *);
+int	pfkey_recv_conf(struct peer *, struct imsg *);
+int	tcp_md5_check(int, struct auth_config *);
+int	tcp_md5_set(int, struct auth_config *, struct bgpd_addr *);
 int	tcp_md5_prep_listener(struct listen_addr *, struct peer_head *);
 void	tcp_md5_add_listener(struct bgpd_config *, struct peer *);
 void	tcp_md5_del_listener(struct bgpd_config *, struct peer *);
@@ -296,13 +302,14 @@ struct rtr_session;
 size_t			 rtr_count(void);
 void			 rtr_check_events(struct pollfd *, size_t);
 size_t			 rtr_poll_events(struct pollfd *, size_t, time_t *);
-struct rtr_session	*rtr_new(uint32_t, char *);
+struct rtr_session	*rtr_new(uint32_t, struct rtr_config_msg *);
 struct rtr_session	*rtr_get(uint32_t);
 void			 rtr_free(struct rtr_session *);
 void			 rtr_open(struct rtr_session *, int);
 void			 rtr_config_prep(void);
 void			 rtr_config_merge(void);
-void			 rtr_config_keep(struct rtr_session *);
+void			 rtr_config_keep(struct rtr_session *,
+			     struct rtr_config_msg *);
 void			 rtr_roa_merge(struct roa_tree *);
 void			 rtr_aspa_merge(struct aspa_tree *);
 void			 rtr_shutdown(void);
@@ -321,7 +328,7 @@ void	rtr_recalc(void);
 RB_PROTOTYPE(peer_head, peer, entry, peer_compare);
 
 void		 session_main(int, int);
-void		 bgp_fsm(struct peer *, enum session_events);
+void		 bgp_fsm(struct peer *, enum session_events, struct ibuf *);
 int		 session_neighbor_rrefresh(struct peer *p);
 struct peer	*getpeerbydesc(struct bgpd_config *, const char *);
 struct peer	*getpeerbyip(struct bgpd_config *, struct sockaddr *);
@@ -331,6 +338,7 @@ int		 imsg_ctl_parent(struct imsg *);
 int		 imsg_ctl_rde(struct imsg *);
 int		 imsg_ctl_rde_msg(int, uint32_t, pid_t);
 void		 session_stop(struct peer *, uint8_t, const char *);
+struct bgpd_addr *session_localaddr(struct peer *);
 
 /* timer.c */
 struct timer	*timer_get(struct timer_head *, enum Timer);
