@@ -1,4 +1,4 @@
-/*	$OpenBSD: util.c,v 1.87 2024/07/03 08:39:43 job Exp $ */
+/*	$OpenBSD: util.c,v 1.89 2024/11/21 13:05:23 claudio Exp $ */
 
 /*
  * Copyright (c) 2006 Claudio Jeker <claudio@openbsd.org>
@@ -32,22 +32,6 @@
 #include "rde.h"
 #include "log.h"
 
-char *
-ibuf_get_string(struct ibuf *buf, size_t len)
-{
-	char *str;
-
-	if (ibuf_size(buf) < len) {
-		errno = EBADMSG;
-		return (NULL);
-	}
-	str = strndup(ibuf_data(buf), len);
-	if (str == NULL)
-		return (NULL);
-	ibuf_skip(buf, len);
-	return (str);
-}
-
 const char *
 log_addr(const struct bgpd_addr *addr)
 {
@@ -161,14 +145,16 @@ const struct ext_comm_pairs iana_ext_comms[] = IANA_EXT_COMMUNITIES;
 const char *
 log_ext_subtype(int type, uint8_t subtype)
 {
-	static char etype[6];
+	static char etype[16];
 	const struct ext_comm_pairs *cp;
 
 	for (cp = iana_ext_comms; cp->subname != NULL; cp++) {
 		if ((type == cp->type || type == -1) && subtype == cp->subtype)
 			return (cp->subname);
 	}
-	snprintf(etype, sizeof(etype), "[%u]", subtype);
+	if (type == -1)
+		return ("???");
+	snprintf(etype, sizeof(etype), "[%hhx:%hhx]", (uint8_t)type, subtype);
 	return (etype);
 }
 
